<?php

namespace EventPortal\Domain\Delegate\Models;

use EventPortal\Concerns\AddToCasts;
use EventPortal\Concerns\AddToFillable;
use EventPortal\Domain\Event\Form\Enums\State;
use EventPortal\Domain\Event\Form\Models\FormData;
use EventPortal\Domain\Event\Form\Models\FormStep;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventDelegate;
use EventPortal\Domain\Form\Form;
use EventPortal\Domain\Model;
use EventPortal\Domain\Table\Concerns\HasTable;
use EventPortal\Domain\Util\Status;
use Illuminate\Database\Eloquent\Relations\BelongsTo;
use Illuminate\Database\Eloquent\Relations\BelongsToMany;
use Illuminate\Database\Eloquent\Relations\HasManyThrough;
use Illuminate\Database\Eloquent\Relations\HasOneThrough;
use OwenIt\Auditing\Auditable;
use OwenIt\Auditing\Contracts\Auditable as AuditableInterface;

class Delegate extends Model implements AuditableInterface
{
    use HasTable, AddToFillable, AddToCasts, Auditable;

    const KEY = 'delegate';

    /** @var array<string> */
    public $with = [
        'user',
    ];

    /** @var array<string> */
    protected $fillable = [
        'user_id',
        'pa_email',
    ];

    public function events(): BelongsToMany
    {
        return $this
            ->belongsToMany(Event::class)
            ->using(EventDelegate::class)
            ->withPivot(['id', 'status'])
            ->withTimestamps();
    }

    public function user(): BelongsTo
    {
        return $this->belongsTo(config('auth.providers.users.model'), 'user_id');
    }

    public function formData(): HasManyThrough
    {
        return $this->hasManyThrough(FormData::class, 'delegate_form_data');
    }

    public function currentStep(): HasOneThrough
    {
        return $this->hasOneThrough(FormStep::class, DelegateFormStep::class, 'form_step_id', 'id')
            ->where('state', State::Partial()->getValue())
            ->orderBy('delegate_form_step.updated_at', 'desc');
    }

    public function nextStep(Form $form): ?FormStep
    {
        $completed = DelegateFormStep::where('delegate_id', $this->id)
            ->where('state', State::Complete()->getValue())
            ->get();

        return FormStep::where('id', $form->steps->pluck('id'))
            ->whereNotIn('id', $completed->pluck('id'))
            ->first();
    }

    public function getReadableStatusAttribute(): string
    {
        return Status::make(EventDelegate::KEY)->getReadable($this->attributes['status']);
    }

    public function getNameAttribute(): string
    {
        return $this->user->first_name . ' ' . $this->user->last_name;
    }

    public function hasPA(): bool
    {
        return $this->pa_email !== null;
    }
}
