<?php

namespace EventPortal\Tests\Unit\Event\Form\Actions;

use EventPortal\Domain\Event\Form\Actions\UpdateQuestions;
use EventPortal\Domain\Event\Form\Enums\GdprType;
use EventPortal\Domain\Event\Form\Enums\QuestionType;
use EventPortal\Domain\Event\Form\Models\FormStepSection;
use EventPortal\Domain\Event\Form\Models\Question;
use EventPortal\Domain\Event\Form\Models\QuestionOption;
use EventPortal\Domain\Event\Form\Models\QuestionValidation;
use EventPortal\Domain\Event\Form\ValueObjects\QuestionData;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Illuminate\Support\Collection;

class UpdateQuestionsTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function can_update_form_question(): void
    {
        $section = factory(FormStepSection::class)->create();
        factory(Question::class)->create(['form_step_section_id' => $section->id]);

        (new UpdateQuestions)->execute($this->getUpdatedQuestion(), $section);

        $question = $section->questions->first();
        $this->assertEquals('Updated test question', $question->question);
        $this->assertEquals('Updated test help', $question->help_text);
        $this->assertEquals('after', $question->help_position);
        $this->assertEquals(GdprType::Private()->getValue(), $question->gdpr_type);
        $this->assertEquals(1, $question->order);
        $this->assertEquals('Updated test uuid', $question->uuid);
        $this->assertEquals(QuestionType::ParagraphTextBox()->getValue(), $question->type);
    }

    private function getUpdatedQuestion(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'id' => 1,
                'question' => 'Updated test question',
                'helpText' => 'Updated test help',
                'helpPosition' => 'after',
                'gdprType' => GdprType::Private()->getValue(),
                'order' => 1,
                'uuid' => 'Updated test uuid',
                'component' => [
                    'name' => [
                        'value' => QuestionType::ParagraphTextBox()->getValue(),
                    ],
                ],
            ],
        ], $override))->mapInto(QuestionData::class);
    }

    /** @test **/
    public function can_add_new_form_question(): void
    {
        $section = factory(FormStepSection::class)->create();

        (new UpdateQuestions)->execute($this->getNewQuestion(), $section);

        $question = $section->questions->first();
        $this->assertEquals('New test question', $question->question);
        $this->assertEquals('New test help', $question->help_text);
        $this->assertEquals('after', $question->help_position);
        $this->assertEquals(GdprType::Sensitive()->getValue(), $question->gdpr_type);
        $this->assertEquals(1, $question->order);
        $this->assertEquals('New test uuid', $question->uuid);
        $this->assertEquals(QuestionType::SingleLineTextBox()->getValue(), $question->type);
    }

    private function getNewQuestion(array $override = []): Collection
    {
        return collect(array_merge([
            [
                'question' => 'New test question',
                'helpText' => 'New test help',
                'helpPosition' => 'after',
                'gdprType' => GdprType::Sensitive()->getValue(),
                'order' => 1,
                'uuid' => 'New test uuid',
                'component' => [
                    'name' => [
                        'value' => QuestionType::SingleLineTextBox()->getValue(),
                    ],
                ],
            ],
        ], $override))->mapInto(QuestionData::class);
    }

    /** @test **/
    public function deletes_unused_form_question(): void
    {
        $section = factory(FormStepSection::class)->create();
        factory(Question::class)->create(['form_step_section_id' => $section->id]);

        (new UpdateQuestions)->execute(collect([]), $section);

        $this->assertCount(0, $section->questions);
    }

    /** @test **/
    public function deletes_option_and_validation_when_deleting_question(): void
    {
        $section = factory(FormStepSection::class)->create();
        $question = factory(Question::class)->create(['form_step_section_id' => $section->id]);
        factory(QuestionOption::class)->create(['question_id' => $question->id]);
        factory(QuestionValidation::class)->create(['question_id' => $question->id]);

        (new UpdateQuestions)->execute(collect([]), $section);

        $this->assertCount(0, QuestionOption::all());
        $this->assertCount(0, QuestionValidation::all());
    }
}
